from .. import WidgetHandlerManager
from .. import AbstractItem
from . import Plasma__IconWidget

class Lancelot__HoverIconHandler(Plasma__IconWidget.Plasma__IconWidgetHandler):
    def name(self):
        return "Lancelot::HoverIcon"

    def include(self):
        includes = "lancelot/widgets/HoverIcon.h plasma/widgets/icon.h".split(" ")
        includesCode = ""
        for include in includes:
            if (include != ""):
                includesCode += "#include<" + include + ">\n"
        return includesCode


    def setup(self):
        setup = Plasma__IconWidget.Plasma__IconWidgetHandler.setup(self)


        if self.hasAttribute('activationMethod'):
            setup += self.attribute('name') \
                  + '->setActivationMethod(' + self.attribute('activationMethod') + ');'


        return setup;

WidgetHandlerManager.addHandler(Lancelot__HoverIconHandler())





from .. import WidgetHandlerManager
from .. import AbstractItem
from . import QGraphicsWidget

class QGraphicsProxyWidgetHandler(QGraphicsWidget.QGraphicsWidgetHandler):
    def name(self):
        return "QGraphicsProxyWidget"

    def include(self):
        includes = "".split(" ")
        includesCode = ""
        for include in includes:
            if (include != ""):
                includesCode += "#include<" + include + ">\n"
        return includesCode


    def setup(self):
        setup = QGraphicsWidget.QGraphicsWidgetHandler.setup(self)



        return setup;

WidgetHandlerManager.addHandler(QGraphicsProxyWidgetHandler())





from .. import WidgetHandlerManager
from .. import AbstractItem
from . import Lancelot__Widget

## Changed area begin: ##
import xml
## Changed area end. ##

class Lancelot__PanelHandler(Lancelot__Widget.Lancelot__WidgetHandler):
    def name(self):
        return "Lancelot::Panel"

    def include(self):
        includes = "lancelot/widgets/Panel.h QIcon QSize QString".split(" ")
        includesCode = ""
        for include in includes:
            if (include != ""):
                includesCode += "#include<" + include + ">\n"
        return includesCode


    def setup(self):
        setup = Lancelot__Widget.Lancelot__WidgetHandler.setup(self)


        if self.hasAttribute('icon'):
            setup += self.attribute('name') \
                  + '->setIcon(KIcon("' + self.attribute('icon') + '"));'

        if self.hasAttribute('iconSize'):
            setup += self.attribute('name') \
                  + '->setIconSize(QSize(' + self.attribute('iconSize') + '));'

        if self.hasAttribute('title'):
            setup += self.attribute('name') \
                  + '->setTitle(i18n("' + self.attribute('title') + '"));'

        if self.hasAttribute('showingTitle'):
            setup += self.attribute('name') \
                  + '->setShowingTitle(' + self.attribute('showingTitle') + ');'
        ## Changed area begin: ##
        for child in self.node().childNodes:
            if not child.nodeType == xml.dom.Node.ELEMENT_NODE or not child.prefix == "item":
                continue

            setup += self.attribute("name") + "->setLayoutItem(" + child.getAttribute("name") + ");\n"
        ## Changed area end. ##


        return setup;

WidgetHandlerManager.addHandler(Lancelot__PanelHandler())





from .. import WidgetHandlerManager
from .. import AbstractItem
from . import Lancelot__CustomListView

class Lancelot__ActionListViewHandler(Lancelot__CustomListView.Lancelot__CustomListViewHandler):
    def name(self):
        return "Lancelot::ActionListView"

    def include(self):
        includes = "lancelot/widgets/ActionListView.h".split(" ")
        includesCode = ""
        for include in includes:
            if (include != ""):
                includesCode += "#include<" + include + ">\n"
        return includesCode


    def setup(self):
        setup = Lancelot__CustomListView.Lancelot__CustomListViewHandler.setup(self)


        if self.hasAttribute('extenderPosition'):
            setup += self.attribute('name') \
                  + '->setExtenderPosition(' + self.attribute('extenderPosition') + ');'

        if self.hasAttribute('categoriesActivable'):
            setup += self.attribute('name') \
                  + '->setCategoriesActivable(' + self.attribute('categoriesActivable') + ');'

        if self.hasAttribute('showsExtendersOutside'):
            setup += self.attribute('name') \
                  + '->setShowsExtendersOutside(' + self.attribute('showsExtendersOutside') + ');'


        return setup;

WidgetHandlerManager.addHandler(Lancelot__ActionListViewHandler())





from .. import WidgetHandlerManager
from .. import AbstractItem
${PYTHON_IMPORT}

class ${PYTHON_NAME}Handler(${PARENT_PYTHON_NAME}.${PARENT_PYTHON_NAME}Handler):
    def name(self):
        return "${NAME}"

    def include(self):
        includes = "${INCLUDES}".split(" ")
        includesCode = ""
        for include in includes:
            if (include != ""):
                includesCode += "#include<" + include + ">\n"
        return includesCode


    def setup(self):
        setup = ${PARENT_PYTHON_NAME}.${PARENT_PYTHON_NAME}Handler.setup(self)

        ${PROPERTY_SETUP_BEGIN}
        if self.hasAttribute('${PROPERTY_NAME}'):
            setup += self.attribute('name') \
                  + '->${PROPERTY_SETTER}(${PROPERTY_VALUE_PREFIX}' + self.attribute('${PROPERTY_NAME}') + '${PROPERTY_VALUE_SUFIX});'
        ${PROPERTY_SETUP_END}

        return setup;

WidgetHandlerManager.addHandler(${PYTHON_NAME}Handler())




#  Copyright (C) 2007, 2008, 2009, 2010 Ivan Cukic <ivan.cukic(at)kde.org>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License version 2,
#  or (at your option) any later version, as published by the Free
#  Software Foundation
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU General Public License for more details
#
#  You should have received a copy of the GNU General Public
#  License along with this program; if not, write to the
#  Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from . import debug

class AbstractItemHandler:
    __node = 0

    def __init__(self):
        self.__node = 0

    """ @returns the full C++ name for this class """
    def name(self):
        return "Abstract"

    """ @returns the XML namespace for this class (for use in the PUCK XML) """
    def namespace(self):
        return self.name().replace(":", "_")

    def setNode(self, node):
        self.__node = node

    def node(self):
        return self.__node

    """ @returns the C++ inlcudes for needed by the class """
    def include(self):
        return "/* " + self.attribute("type") + " declaration - not implemented */"

    """ @returns the C++ variable declaration """
    def declaration(self):
        return self.attribute("type") + " * " + self.attribute("name") + ";"

    """ @returns the C++ variable initialization """
    def initialization(self):
        return self._construction() + ";";

    def _construction(self):
        return self.attribute("name") + " = new " + self.attribute("type") + "()"

    """ @returns the C++ object setup (properties etc.) """
    def setup(self):
        return ""

    """ @returns whether the active node has the specified attribute """
    def hasAttribute(self, attrName):
        debug.message(self.name(), " wants attribute " + self.namespace() + ":" + attrName)
        return \
            self.node().hasAttribute(self.name() + ":" + attrName) or \
            self.node().hasAttribute(attrName)

    """ @returns the value of the active node's attribute """
    def attribute(self, attrName):
        debug.message(self.name(), " retrieves attribute " + self.namespace() + ":" + attrName)

        result = None

        if self.node().hasAttribute(self.name() + ":" + attrName):
            result = self.node().getAttribute(self.name() + ":" + attrName)

        elif self.node().hasAttribute(attrName):
            result = self.node().getAttribute(attrName)

        if not result == None:
            debug.message(" return ", result)
        else:
            debug.message(" return ", " NONE ")

        return result



from .. import LayoutHandlerManager
from .. import AbstractItem

class AbstractLayoutHandler(AbstractItem.AbstractItemHandler):

    def __init__(self):
        self.__node = 0

    def setup(self):
        setup = ""

        # if self.hasAttribute("margin"):
        #     setup += self.attribute("name") + "->setMargin(" + self.attribute("margin") + ");\n";

        # if self.hasAttribute("spacing"):
        #     setup += self.attribute("name") + "->setSpacing(" + self.attribute("spacing") + ");\n";
        return setup

LayoutHandlerManager.addHandler(AbstractLayoutHandler())



from .. import WidgetHandlerManager
from .. import AbstractItem


class QGraphicsLayoutItemHandler(AbstractItem.AbstractItemHandler):
    def name(self):
        return "QGraphicsLayoutItem"

    def include(self):
        includes = "".split(" ")
        includesCode = ""
        for include in includes:
            if (include != ""):
                includesCode += "#include<" + include + ">\n"
        return includesCode


    def setup(self):
        setup = AbstractItem.AbstractItemHandler.setup(self)


        if self.hasAttribute('minimumSize'):
            setup += self.attribute('name') \
                  + '->setMinimumSize(QSize(' + self.attribute('minimumSize') + '));'

        if self.hasAttribute('minimumWidth'):
            setup += self.attribute('name') \
                  + '->setMinimumWidth(' + self.attribute('minimumWidth') + ');'

        if self.hasAttribute('minimumHeight'):
            setup += self.attribute('name') \
                  + '->setMinimumHeight(' + self.attribute('minimumHeight') + ');'

        if self.hasAttribute('maximumSize'):
            setup += self.attribute('name') \
                  + '->setMaximumSize(QSize(' + self.attribute('maximumSize') + '));'

        if self.hasAttribute('maximumWidth'):
            setup += self.attribute('name') \
                  + '->setMaximumWidth(' + self.attribute('maximumWidth') + ');'

        if self.hasAttribute('maximumHeight'):
            setup += self.attribute('name') \
                  + '->setMaximumHeight(' + self.attribute('maximumHeight') + ');'

        if self.hasAttribute('preferredSize'):
            setup += self.attribute('name') \
                  + '->setPreferredSize(QSize(' + self.attribute('preferredSize') + '));'

        if self.hasAttribute('preferredWidth'):
            setup += self.attribute('name') \
                  + '->setPreferredWidth(' + self.attribute('preferredWidth') + ');'

        if self.hasAttribute('preferredHeight'):
            setup += self.attribute('name') \
                  + '->setPreferredHeight(' + self.attribute('preferredHeight') + ');'


        return setup;

WidgetHandlerManager.addHandler(QGraphicsLayoutItemHandler())





#!/usr/bin/env python

#
#  Copyright (C) 2007, 2008, 2009, 2010 Ivan Cukic <ivan.cukic(at)kde.org>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License version 2,
#  or (at your option) any later version, as published by the Free
#  Software Foundation
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU General Public License for more details
#
#  You should have received a copy of the GNU General Public
#  License along with this program; if not, write to the
#  Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

import sys
import os
import re

# global variables
namespace = ""
className = ""
parentClass = ""
properties = []
includes = []

def parseFile(inputFile):
    global namespace
    global className
    global parentClass
    global properties
    global includes

    namespace = ""
    className = ""
    parentClass = ""
    properties = []
    includes = []

    reNamespace = re.compile("namespace\s+([^\s]+)")
    reClass     = re.compile("class.*[\s]+([^\s]+)[\s]*:[\s]*public[\s]*([A-Za-z::]+)?|class.*[\s]+([^\s]+)[\s]*({|;|$)")
    reProperty  = re.compile("Q_PROPERTY\s*\(\s*([^\s]+)\s+([^\s]+)\s+READ\s+([^\s]+)\s+WRITE\s+([^\s]+)\s*\)")
    reIncludes  = re.compile("(\/\/\s*@puck\s*)?L_INCLUDE\s*\(\s*(.+)\s*\)")
    reExtra     = re.compile(".*L_EXTRA\s*\(\s*(.+)\s*\)")

    data = open(inputFile).readlines()

    for line in data:
        line = line.strip()

        match = reNamespace.match(line)
        if match:
            namespace = match.group(1)
            continue

        match = reClass.match(line)
        if match:
            print "-------"
            if className != "":
                processClass()

            properties = []
            includes = ""

            print match.groups()
            if match.group(1) != None:
                className = match.group(1)
                parentClass = match.group(2)
            else:
                className = match.group(3)
                parentClass = ""
            print "Found class '" + className + "' inherits '" + parentClass + "'"
            reConstructor = re.compile("^" + className + "\((.*)\);")
            reConstructorBegin = re.compile(className)
            continue

        match = reProperty.match(line)
        if match:
            print "Found property ", match.groups()
            append = list(match.groups())

            match = reExtra.match(line)
            if match:
                append.append(match.group(1).strip())

            properties.append(append)
            # print properties
            continue

        match = reIncludes.match(line)
        if match:
            print "Debug ", match
            print "Found includes ", match.group(2)
            includes = match.group(2)
            print includes
    if className != "":
        processClass()

def constructPythonName(fullClass):
    result = fullClass.replace("::", "__")
    return result.strip("_")

def constructCPPName(fullClass):
    return fullClass.strip(":")

def propertyValueExes(type):
    # Qt and Kde types
    if (type == "QString"):
        return "\"${VALUE}\""
    elif (type == "QString i18n"):
        return "i18n(\"${VALUE}\")"
    elif (type == "KIcon"):
        return "KIcon(\"${VALUE}\")"
    elif (type == "QIcon"):
        return "KIcon(\"${VALUE}\")"

    # Lancelot types
    elif (type == "ExtenderPosition"):
        return "Lancelot::${VALUE}Extender"
    elif (type == "ActivationMethod"):
        return "Lancelot::${VALUE}Activate"

    # Heuristics
    elif (type.find("::") != -1):
        position = type.find("::");
        return type[0:position] + "::${VALUE}"
    elif (type.startswith("Q")):
        return type + "(${VALUE})"
    else:
        return "${VALUE}"

def processClass():
    global namespace
    global className
    global parentClass
    global properties
    global includes

    fullClass = namespace + "::" + className
    outputFile = constructPythonName(fullClass) + ".py"
    print "Generating", outputFile

    template = open(os.path.join(os.path.dirname(os.path.abspath(__file__)), "template/widget.py"))
    template = template.readlines()
    template = "".join(template)

    if (parentClass != ""):
        parentPythonName = constructPythonName(parentClass)
        pythonImport = "from . import " + parentPythonName;
    else:
        parentPythonName = "AbstractItem"
        pythonImport = ""

    template = template\
            .replace("${NAME}", constructCPPName(fullClass))\
            .replace("${INCLUDES}", includes)\
            .replace("${PYTHON_NAME}", constructPythonName(fullClass))\
            .replace("${PYTHON_IMPORT}", pythonImport)\
            .replace("${PARENT_PYTHON_NAME}", parentPythonName)

    begin = template.find("${PROPERTY_SETUP_BEGIN}") # 23 length
    end   = template.find("${PROPERTY_SETUP_END}")   # 21 length
    propertyTemplate = template[begin + 23:end]
    propertiesCode = ""
    for property in properties:
        exes = ""
        print "Has this no of props: ", len(property);
        if (len(property) == 5):
            exes = propertyValueExes(property[0] + " " + property[4]).split("${VALUE}")
        else:
            exes = propertyValueExes(property[0]).split("${VALUE}")
        print exes
        propertiesCode += propertyTemplate\
                .replace("${PROPERTY_NAME}", property[1])\
                .replace("${PROPERTY_SETTER}", property[3])\
                .replace("${PROPERTY_VALUE_PREFIX}", exes[0])\
                .replace("${PROPERTY_VALUE_SUFIX}", exes[1])

    template = template[0:begin] + propertiesCode + template[end + 21:]
    # print template

    reValidFile = re.compile("^[A-Za-z0-9_.]*$")
    match = reValidFile.match(outputFile)
    if match:
        output = open(outputFile, 'w')
        print >> output, template
        output.close()

######## MAIN PROGRAM ###########

inputFile  = sys.argv[1]
# outputFile = sys.argv[2]

print "Parsing: " + inputFile
# print "And creating PUCK module from it: " + outputFile

parseFile(inputFile)




from .. import WidgetHandlerManager
from .. import AbstractItem
from . import QGraphicsProxyWidget

class Plasma__ScrollBarHandler(QGraphicsProxyWidget.QGraphicsProxyWidgetHandler):
    def name(self):
        return "Plasma::ScrollBar"

    def include(self):
        includes = "".split(" ")
        includesCode = ""
        for include in includes:
            if (include != ""):
                includesCode += "#include<" + include + ">\n"
        return includesCode


    def setup(self):
        setup = QGraphicsProxyWidget.QGraphicsProxyWidgetHandler.setup(self)


        if self.hasAttribute('singleStep'):
            setup += self.attribute('name') \
                  + '->setSingleStep(' + self.attribute('singleStep') + ');'

        if self.hasAttribute('pageStep'):
            setup += self.attribute('name') \
                  + '->setPageStep(' + self.attribute('pageStep') + ');'

        if self.hasAttribute('stylesheet'):
            setup += self.attribute('name') \
                  + '->setStyleSheet("' + self.attribute('stylesheet') + '");'


        return setup;

WidgetHandlerManager.addHandler(Plasma__ScrollBarHandler())


